{

module FilterParser3 where
import Regexp
import Char
import HXML
}

%name filterparse
%tokentype { Token }
%monad { P } { thenP } { returnP }
%lexer { tokenList } { TokenEOF }

%token
	"count"			{ TokenCount }
	"filterBest"    	{ TokenFilterBest }
	"filterAll"     	{ TokenFilterAll }
	"filterBestExact"	{ TokenFilterBExact }
	"filterAllExact"	{ TokenFilterAExact }
	"file"	  		{ TokenFile }
	'$'	  		{ TokenType }
	var	  		{ TokenVar $$ }
	num			{ TokenNum $$ }
	text	  		{ TokenText $$ }
	tag	  		{ TokenTag $$ }
	cond	  		{ TokenCond $$ }
	"in"	  		{ TokenIn }
	"and"	  		{ TokenAnd $$ }
	"or"	  		{ TokenOr $$ }
	"xor"	  		{ TokenXor $$ }
	"where"   		{ TokenWhere }
	"(P)"	  		{ TokenP $$}
	"(N)"	  		{ TokenN $$}
	'('	  		{ TokenOB }
	')'	  		{ TokenCB }
	','	  		{ TokenComma }
	'|'	  		{ TokenBar }
	'?'	  		{ TokenQmark }

%%
Rule : Filter ExternalTerm "in" Expression Label					{ Tree (ELNode "rule" []) [$1, $2,$4,(Tree (TXNode "conditions") []),(Tree (TXNode $5) []), (Tree (TXNode "-1") [])] }
      |Filter ExternalTerm "in" Expression "where" Conditions Label			{ Tree (ELNode "rule" []) [$1, $2,$4,(Tree (ELNode "conditions" []) $6),(Tree (TXNode $7) []), (Tree (TXNode "-1") [])] }
      |"count" num '(' Filter ExternalTerm "in" Expression Label ')'			{ Tree (ELNode "rule" []) [$4, $5,$7,(Tree (TXNode "conditions") []),(Tree (TXNode $8) []), (Tree (TXNode $2) [])] }
      |"count" num '(' Filter ExternalTerm "in" Expression "where" Conditions Label ')' { Tree (ELNode "rule" []) [$4, $5,$7,(Tree (ELNode "conditions" []) $9),(Tree (TXNode $10) []), (Tree (TXNode $2) [])] }

Filter :  "filterBest"		{ Tree (TXNode "best") [] }
	| "filterAll"		{ Tree (TXNode "all") [] }
	| "filterBestExact" 	{ Tree (TXNode "bestEx") [] }
	| "filterAllExact" 	{ Tree (TXNode "allEx") [] }

Expression : GroundTerm	   { Tree (ELNode "xml" []) [Tree (TXNode $1) []] }
	  | "file" text	   { Tree (ELNode "file" []) [Tree (TXNode $2) []] }
	  | '(' Rule ')'   { $2 }

Conditions : cond  			{ [(Tree (TXNode $1) [])] }
      	   | cond ',' Conditions 	{ (Tree (TXNode $1) []) : $3 }

Label : {- empty -}	{ "P" }
      | "(P)"		{ "P" }
      | "(N)"		{ "N" }

ExternalTerm : Term					{ $1 }
	     | Operator '(' ExternalTermList ')' 	{ Tree (ELNode $1 []) $3 }

ExternalTermList : ExternalTerm				{ [$1] }
		 | ExternalTerm ',' ExternalTermList	{ $1:$3 }

Operator : "and"	{ "and" }
	 | "or"		{ "or" }
	 | "xor"	{ "xor" }

Term : text             		{ Tree (TXNode $1) [] }
      |tag '(' var ')'			{ Tree (ELNode $1 [("var",$3)]) [] }
      |'$' tag '(' var ')'		{ Tree (ELNode $2 [("var",$4),("type","E")]) [] }
      |tag '(' ')'			{ Tree (ELNode $1 []) [] }
      |'$' tag '(' ')'			{ Tree (ELNode $2 [("type","E")]) [] }
      |tag '(' Term ')'			{ Tree (ELNode $1 []) [$3] }
      |'$' tag '(' Term ')'		{ Tree (ELNode $2 [("type","E")]) [$4] }
      |tag '(' Termlist ')' 		{ Tree (ELNode $1 []) [(Tree (ELNode "and" []) $3)] }
      |'$' tag '(' Termlist ')' 	{ Tree (ELNode $2 [("type","E")]) [(Tree (ELNode "and" []) $4)] }
      |tag '(' TermOrlist ')' 		{ Tree (ELNode $1 []) [(Tree (ELNode "or" []) $3)] }
      |'$' tag '(' TermOrlist ')' 	{ Tree (ELNode $2 [("type","E")]) [(Tree (ELNode "or" []) $4)] }
      |tag '(' TermXorlist ')' 		{ Tree (ELNode $1 []) [(Tree (ELNode "xor" []) $3)] }
      |'$' tag '(' TermXorlist ')' 	{ Tree (ELNode $2 [("type","E")]) [(Tree (ELNode "xor" []) $4)] }

GroundTerm : text			{ $1 }
	   | tag '(' ')'		{ "<"++$1++"/>" }
      	   | tag '(' GroundTermList ')' { "<"++$1++">"++$3++"</"++$1++">" }

Termlist : Term					{ $1 : [] }
	 | Term ',' Termlist  			{ $1 : $3 }
      	 | Term ',' '(' TermOrlist ')'  	{ $1 : [(Tree (ELNode "or" []) $4)] }
      	 | Term ',' '(' TermXorlist ')'  	{ $1 : [(Tree (ELNode "xor" []) $4)] }
      	 | '(' TermOrlist ')' ',' Termlist  	{ (Tree (ELNode "or" []) $2) : $5 }
      	 | '(' TermXorlist ')' ',' Termlist  	{ (Tree (ELNode "xor" []) $2) : $5 }

TermOrlist : Term				{ $1 : [] }
	   | Term '|' TermOrlist  		{ $1 : $3 }
      	   | Term '|' '(' Termlist ')'  	{ $1 : [(Tree (ELNode "and" []) $4)] }
      	   | Term '|' '(' TermXorlist ')'  	{ $1 : [(Tree (ELNode "xor" []) $4)] }
      	   | '(' Termlist ')' '|' TermOrlist  	{ (Tree (ELNode "and" []) $2) : $5 }
      	   | '(' TermXorlist ')' '|' TermOrlist { (Tree (ELNode "xor" []) $2) : $5 }
      	  
TermXorlist : Term					{ $1 : [] }
	    | Term '?' TermXorlist  			{ $1 : $3 }
      	    | Term '?' '(' Termlist ')'  		{ $1 : [(Tree (ELNode "and" []) $4)] }
      	    | Term '?' '(' TermOrlist ')'  		{ $1 : [(Tree (ELNode "or" []) $4)] }
      	    | '(' Termlist ')' '?' TermXorlist  	{ (Tree (ELNode "and" []) $2) : $5 }
      	    | '(' TermOrlist ')' '?' TermXorlist  	{ (Tree (ELNode "or" []) $2) : $5 }

GroundTermList : GroundTerm  	   			{ $1 }
	       | GroundTerm ',' GroundTermList  	{ $1++$3 }

{
data ParseResult a
	= ParseOk a
	| ParseFail String

type P a = String -> Int -> ParseResult a

thenP :: P a -> (a -> P b) -> P b
thenP m k = \s l -> 
	case m s l of
		ParseFail s -> ParseFail s
		ParseOk a -> k a s l

returnP :: a -> P a
returnP a = \s l -> ParseOk a

happyError :: P a
happyError = \s i -> ParseFail ("Parse error at line " ++ show (i::Int) ++ "\n")

data Term = XML
	deriving Show

data Token
	= TokenVar String
	| TokenText String
	| TokenTag String
	| TokenCond String
	| TokenNum String
	| TokenCount
	| TokenFilterBest
	| TokenFilterAll
	| TokenFilterBExact
	| TokenFilterAExact
	| TokenFile
	| TokenType
	| TokenIn
	| TokenAnd String
	| TokenOr String
	| TokenXor String
	| TokenWhere
	| TokenP String
	| TokenN String
	| TokenOB
	| TokenCB
	| TokenComma
	| TokenBar
	| TokenQmark
	| TokenEOF
  deriving Show

tokenList :: (Token -> P a) -> P a
tokenList cont xs = case xs of
	[] -> cont TokenEOF []
  	('\n':cs) -> \line -> tokenList cont cs (line+1)
  	(c:cs) | isSpace c -> tokenList cont cs
	otherwise
		|token==[] -> cont TokenEOF []
		|token=="(" -> cont TokenOB (tail xs)
		|token=="," -> cont TokenComma (tail xs)
		|token=="|" -> cont TokenBar (tail xs)
		|token=="?" -> cont TokenQmark (tail xs)
		|token==")" -> cont TokenCB (tail xs)
		|token=="$" -> cont TokenType (tail xs)
		|token=="count" -> cont TokenCount (drop (length token) xs)
		|token=="file" -> cont TokenFile (drop (length token) xs)
		|token=="filterBest" -> cont TokenFilterBest (drop (length token) xs)
		|token=="filterAll" -> cont TokenFilterAll (drop (length token) xs)
		|token=="filterBestExact" -> cont TokenFilterBExact (drop (length token) xs)
		|token=="filterAllExact" -> cont TokenFilterAExact (drop (length token) xs)
		|token=="in" -> cont TokenIn (drop (length token) xs)
		|token=="and" -> cont (TokenAnd "and") (drop (length token) xs)
		|token=="or" -> cont (TokenOr "or") (drop (length token) xs)
		|token=="xor" -> cont (TokenXor "xor") (drop (length token) xs)
		|token=="where" -> cont TokenWhere (drop (length token) xs)
		|token=="(N)" -> cont (TokenN "N") (drop (length token) xs)
		|token=="(P)" -> cont (TokenP "P") (drop (length token) xs)
		|isVar token -> cont (TokenVar token) (drop (length token) xs)
		|isTag token -> cont (TokenTag token) (drop (length token) xs)
		|isNum token -> cont (TokenNum token) (drop (length token) xs)
		|isText token -> cont (TokenText (clean (tail token))) (drop (length token) xs)
		|isCond token -> cont (TokenCond (clean (tail token))) (drop (length token) xs)
		|True -> cont TokenEOF []
	where token = isToken xs

matchRe s re = wholeMatch( searchS re [] s)

clean [x] = []
clean (x:xs) = x:(clean xs)

isToken [] = []
isToken s = matchRe s "count|filterAllExact|filterBestExact|filterBest|filterAll|file|where|and|or|xor|\\(N\\)|\\(P\\)|[a-zA-Z0-9\\[\\]\\_]*|['][a-zA-Z0-9\\:\\/\\s\\.\\_]*[']|[&][a-zA-Z0-9\\\\\\+\\s\\.\\_\\(\\)\\,\\->\\:\\=\\+\\-\\/\\<\\>\\$\\[\\]\\*\\']*[&]|in|\\(|\\$|\\||\\?|\\)|,|\\->|\\:|\\.|\\_|\\<|\\>"

isNum (x:xs) = (wholeMatch(searchS "[0-9]+" [] (x:xs))) == (x:xs)
isNUm (_) = False

isVar (x:xs) = isUpper x
isVar (_) = False

isText (x:xs) =  (x=='\'')
isText (_) = False

isTag (x:xs) = isLower x
isTag (_) = False

isCond (x:xs) = (x=='&')
isCond (_) = False

filterParser s = case filterparse s 1 of
		ParseOk e -> ParseOk e
		ParseFail s -> ParseFail s

type Parse = P XML
filterparse :: Parse
}