module Main where

import HXML
import FilterParser
import ExternalQuery
import Regexp
import Time
import System

main = do{
	args<-getArgs;
	query<-return(head args);
	filterXML query;
	}
	
filterXML s = do {
	x <- return (filterParser s);
	case x of
		ParseOk (Tree (ELNode "rule" []) args) -> do {
			t1<-getClockTime;
			result<-extractElem args 1;
			putStrLn (showFormattedXML 0 1 result);
			t2<-getClockTime;
		        putStrLn ("Ricerca embedding, tempo impiegato:");
		        print (diffClockTimes t2 t1);
			return();
		}
		ParseFail err -> putStrLn ("Attention "++err);
	}

defaultSearchType = "E"

manageFilter::Int->String->Tree XMLNode->Tree XMLNode->([(String,String)],[([String],String)],[String])->Tree XMLNode->Int->IO (Tree XMLNode)
manageFilter nes filter pattern exp cond (Tree (TXNode t) []) n = case exp of
	(Tree (ELNode "rule" []) args) -> do{ newText<-extractElem args 0;
					      return (evalExternalQuery filter defaultSearchType (showFormattedXML 0 0 newText) cond t nes pattern n)}
	(Tree (ELNode "term" []) [(Tree (TXNode text) [])]) -> do{ return (evalExternalQuery filter defaultSearchType text cond t nes pattern n)}
	(Tree (ELNode "file" []) [(Tree (TXNode label) [])]) -> do{  content<-readFile label;
			  					     return (evalExternalQuery filter defaultSearchType content cond t nes pattern n)}

extractElem::[Tree XMLNode]->Int->IO (Tree XMLNode)
extractElem args nesting = do tree<-(manageFilter nesting (getLabel (head args)) (head (tail args)) (head (tail (tail args))) (parseCond (head (tail (tail (tail args))))) (head (tail (tail (tail (tail args))))) (read (getLabel (head (tail (tail (tail (tail (tail args)))))))))
		              return tree

getLabel (Tree (TXNode label) []) = label

parseCond::Tree XMLNode->([(String,String)],[([String],String)],[String])
parseCond (Tree (TXNode "conditions") []) = ([],[],[])
parseCond (Tree (ELNode "conditions" []) args) = parseCondAux args [] [] []

parseCondAux [] rExp cond vars = (rExp,cond,vars)
parseCondAux ((Tree (TXNode lab) []):ts) rExp cond vars
	|t=="R" = parseCondAux ts (((head v),s):rExp) cond (addDistinctVars v vars)
	|t=="N" = parseCondAux ts rExp ((v,s):cond) (addDistinctVars v vars)
	where
	(t,v,s) = parseLabel lab

parseLabel s 
	|matchedAny m = ("R",[delSuffixBlank(beforeMatch m)],delPrefixBlank(afterMatch m))
	|otherwise = ("N",vars,s)
	where
	m = searchS "match" [] s
	vars = searchVars s []
	delPrefixBlank [] = []
	delPrefixBlank (x:xs)
		|x==' ' = delPrefixBlank xs
		|otherwise = (x:xs)
	delSuffixBlank [] = []
	delSuffixBlank (x:xs)
		|x==' ' = []
		|otherwise = x:(delSuffixBlank xs)

searchVars [] v = v
searchVars s v
	|(length nv) == 0 = v
	|otherwise = searchVars (afterMatch m) (nv:v)
	where
	m = searchS "[A-Z]" [] s
	nv = wholeMatch m

addDistinctVars [] vars = vars
addDistinctVars v [] = v
addDistinctVars (v:vs) vars = addDistinctVars vs (addDVar v vars)
	where
	addDVar v [] = [v]
	addDVar v (dv:dvs)
		|v==dv = (dv:dvs)
		|otherwise = dv:(addDVar v dvs)

sepBln::Int
maxLengthElement::Int
sepBln = 3
maxLengthElement = 80

blancos 0 = []
blancos n = " ":(blancos (n-1))

attrFormat (x,y) =[(" ")++x++("=")++("\"")++y++("\"")]

showFormattedXML::Int->Int->Tree XMLNode->String
showFormattedXML prof i (Tree RTNode trees) = showFormattedXML 0 i (Tree (ELNode "Result" []) trees)
showFormattedXML prof i (Tree (ELNode name attr) []) = (concat(blancos (prof*sepBln)))++("<")++(control name i)++(concat(concat(map attrFormat attr)))++("/>\n")
showFormattedXML prof i (Tree (TXNode text) []) = (concat(blancos (prof*sepBln)))++(text)++("\n")
showFormattedXML prof i (Tree (ENNode text) []) = (concat(blancos (prof*sepBln)))++(text)++("\n")
showFormattedXML prof i (Tree (CXNode text) []) = (concat(blancos (prof*sepBln)))++(text)++("\n")
showFormattedXML prof i (Tree (ELNode name attr) [Tree (TXNode text)[]]) = if (((2*(length name))+(length text)+5+(prof*sepBln))>maxLengthElement) 
									 then (concat(blancos (prof*sepBln)))++("<")++(control name i)++(concat(concat(map attrFormat attr)))++(">\n")++(concat(blancos ((prof+1)*sepBln)))++text++("\n")++(concat(blancos (prof*sepBln)))++("</")++(control name i)++(">\n")
									 else (concat(blancos (prof*sepBln)))++("<")++(control name i)++(concat(concat(map attrFormat attr)))++(">")++text++("</")++(control name i)++(">\n")
		
showFormattedXML prof i (Tree (ELNode name attr) args) = (concat(blancos (prof*sepBln)))++("<")++(control name i)++(concat(concat(map attrFormat attr)))++(">\n")++(concat(map (showFormattedXML (prof+1) i) args))++(concat(blancos (prof*sepBln)))++("</")++(control name i)++(">\n")				 

control s i
	|i==0 = s
	|(i==1) && (s=="0000") = "Results"
	|(i==1) && ((s=="and") || (s=="or") || (s=="xor")) = "ResultQuery"
	|otherwise = s