-- $Id: mkSite.hs,v 1.2 2002/03/06 20:23:06 joe Exp $

module Main where

import IO
import qualified System
import XMLCombinators

outputDirectory :: String
outputDirectory = "htdocs"

-- Some utilities:

hRename :: String -> Filter XML XML
hRename  gi = mkElement gi aZero qChildren

hRenameC :: Name -> String -> Filter XML XML
hRenameC gi classatt =
	mkElement gi (mkAtt "class" (aConst classatt)) qChildren

emptyElement :: Name -> Filter a (Name,String) -> Filter a XML
emptyElement name atts = mkElement name atts aZero

hElement :: Name -> String -> Filter XML XML -> Filter XML XML
hElement gi classatt body = mkElement gi (mkAtt "class" (aConst classatt)) body

-- Main program:

main :: IO ()
main = do
    args <- System.getArgs
    case args of
    	[]	->
	    getContents >>= 
	    	printPages (makeIntro +++ makePages) . parseXML
	[fn]	->
	    readFile fn >>= printPages makeIntro . parseXML
	 >> readFile fn >>= printPages makePages . parseXML

printPages :: Filter XML (String,XML) -> XML -> IO ()
printPages f = sequence_ . map (uncurry printPage) . runFilter f

printPage :: String -> XML -> IO ()
printPage name xml =
    let   filename = outputDirectory ++ "/" ++ name ++ ".html"
    in    hPutStrLn stderr ("Writing " ++ filename)
       >> writeFile filename (showXML xml)

makeIntro, makePages :: Filter XML (String,XML)
makePages = qOuter (qElem "page") >>> qAttval "id" &&& htmlPage toHTML
makeIntro = qOuter (qElem "site") >>> aConst "index" &&& htmlPage (
	(qChildElem "title" >>> hElement "H1" "title" keepText)
    +++ (qChildElem "intro" >>> qChildren >>> toHTML)
    +++ mkElem "H2" (mkLiteral "Contents")
    +++ hElement "UL" "toc" (
	    qDescendants >>>
	    qElem "page" >>>
	    hElement "LI" "toc" (
		mkElement "A"
		    (mkAtt "href" (qAttval "id" >>> arr (++".html")))
		    (qChildElem "title" >>> keepText))))

htmlPage :: Filter XML XML -> Filter XML XML
htmlPage body = html where
    html = mkElem "HTML" (mkElem "HEAD" head +++ mkElem "BODY" body)
    head = mkElem "TITLE" (qChildElem "title" >>> keepText)
	+++ emptyElement "LINK" (
		aConst ("rel","stylesheet")
	    +++ aConst ("type","text/css")
	    +++ aConst ("href","hxml.css"))

keepElems :: [Name]
keepElems = ["p","ul","ol","li","dl","dt","dd", "a"]

toHTML, convertNode :: Filter XML XML
toHTML = aFoldTree convertNode
convertNode =
    	qElems keepElems	|?> arr id
    :>	qElem "h"		|?> hRename  "H2"
    :>	qElem "item"		|?> hRename  "LI"
    :>	qElem "t"		|?> hRename  "TT"
    :>	qElem "v"		|?> hRename  "VAR"
    :>	qElem "lit"		|?> hRename  "SAMP"
    :>	qElem "emph"		|?> hRename  "EM"
    :>	qElem "samp"		|?> hRename  "SAMP"
    :>	qElem "para"		|?> hRenameC "DIV" "paragraph"
    :> 	qElem "title"		|?> hRenameC "H1" "title"
    :>	qElem "example"		|?> hRenameC "PRE" "example"
    :>	qElem "syntax"		|?> hRenameC "PRE" "syntax"
    :>	qElem "listing"		|?> hRenameC "PRE" "listing"
    :>	qElem "url"		|?>
    	mkElement "A"
	    (mkAtt "href" (arr stringValue))
	    (mkLiteral "<URL:" +++ keepText +++ mkLiteral ">")
    :>	qElem "email"		|?>
    	mkElement "A"
	    (mkAtt "href" (arr (("mailto:"++) . stringValue)) )
	    keepText
    :>	qElem "changelog"	|?> hRename "DL"
    :>	qElem "change"		|?>
    	(	(mkElem "dt" $ mkElem "i"  $ (qAttval "date" >>> mkText))
	 +++ 	(mkElem "dd" $ mkElem "ul" $ qChildren >>> qElem "LI")
	)
    :>	qText			|?> arr id
    :>	qEntity			|?> arr id
    :>	qChildren

-- *EOF*
